module MonitoredProcess

	class Base < ActiveRecord::Base
		self.table_name = 'monitored_processes'
		
		attr_accessor :log_lambda

	  STATE = { :running => 1, :finished => 2, :finished_with_errors => 3 }

	  # VALIDATIONS
	  validates_presence_of :name, :state, :pid

	  def self.start_process
	    p = MonitoredProcess::Base.new
	    p.name = self.to_s
	    p.state = STATE[:running]
	    p.pid = Process.pid
	    p.console_output = ""
	    p.save!

	    p.log_lambda = lambda do |message|
	      puts "#{DateTime.now} - #{message}"
	    end

	    return p
	  end

	  def self.last_time_executed
	    p = MonitoredProcess.where(:name => self.to_s).last
	    return p.updated_at if !p.blank?
	  end

	  def self.is_running_by_name? (process_name)
	    processes = MonitoredProcess.where(:name => process_name, :state => STATE[:running])
	    return false if processes.blank?

	    is_running = false
	    processes.each do |process|
	      begin
	        Process.getpgid(process.pid)
	        is_running = true
	      rescue Errno::ESRCH
	        process.finish_with_errors! "Unknown error / Process was terminated"
	      end
	    end
	    return is_running
	  end

	  def self.is_running?
	    return self.is_running_by_name? (self.name)
	  end

	  def finish!
	    set_finished_status
	    self.save!
	  end

	  def finish
	    set_finished_status
	    return self.save
	  end

	  def finish_with_errors! (error_description)
	    self.state = STATE[:finished_with_errors]
	    self.error_description = error_description
	    self.calculate_elapsed_seconds
	    self.save!
	  end

	  def calculate_elapsed_time
	    begin
	      hours_with_fraction = self.elapsed_seconds / 3600.0
	      hours = hours_with_fraction.truncate
	      minutes_with_fraction = (hours_with_fraction - hours) * 60
	      minutes = minutes_with_fraction.truncate
	      seconds_with_fraction = (minutes_with_fraction - minutes) * 60
	      seconds = seconds_with_fraction.truncate

	      self.elapsed_time = "#{hours.to_s.rjust(2, "0")}:#{minutes.to_s.rjust(2, "0")}:#{seconds.to_s.rjust(2, "0")}"
	    rescue
	      self.elapsed_time "00:00:00"
	    end
	  end

	  def calculate_elapsed_seconds
	    self.elapsed_seconds = Time.parse(DateTime.now.to_s) - Time.parse(self.created_at.to_s)

	    # Set elapsed time
	    self.calculate_elapsed_time
	  end

	  def output(text)
	    self.console_output += text + "\n"
	    log_lambda.call(text) if !log_lambda.blank?
	  end

	  def log(text)
	    log_lambda.call(text) if !log_lambda.blank?
	  end

	  def send_status_email
	    MonitoredProcessMailer.process_execution_result_email(self).deliver
	  end

	  def self.list(params)
	    parameters = []
	    parameters = params.clone if !params.blank?
	    conditions = []

	    if !params.blank?
	      if !params[:start_date].blank?
	        conditions << "created_at >= :start_date"
	      end
	      if !params[:end_date].blank?
	        conditions << "created_at <= :end_date"
	      end
	      if !params[:process_type].blank?
	        conditions << "name <= :process_type"
	      end
	    end

	    parameters[:sort] = "id_desc" if params[:sort].blank?

	    where(conditions.join(" and "), parameters).sorted(parameters[:sort]).paginate(:page => parameters[:page], :per_page => 100)
	  end

	  def self.translate_state(state)
	    case state
	    when STATE[:running]
	      "Running"
	    when STATE[:finished]
	      "Finished"
	    when STATE[:finished_with_errors]
	      "Finished with errors"
	    end
	  end

	  def self.get_process_names
	    MonitoredProcess.uniq.pluck(:name)
	  end

	  private
	  def set_finished_status
	    self.state = STATE[:finished]
	    self.calculate_elapsed_seconds
	  end
	end

end